package pps.core.common.excel.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;
import pps.core.common.enums.BusinessError;
import pps.core.common.excel.config.ExcelSelected;
import pps.core.common.excel.config.ExcelSelectedResolve;
import pps.core.common.excel.config.SelectedSheetWriteHandler;
import pps.core.common.excel.listener.ExcelListener;
import xstartup.base.exception.XServiceException;
import xstartup.data.XUploadFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * 简单excel实用程序
 *
 * @author ZWT
 * @date 2024/05/07
 */
@Slf4j
public class EasyExcelUtil {

    /**
     * 创建即将导出的sheet页（sheet页中含有带下拉框的列）
     *
     * @param head      导出的表头信息和配置
     * @param sheetNo   sheet索引
     * @param sheetName sheet名称
     * @param <T>       泛型
     * @return sheet页
     */
    public static <T> WriteSheet writeSelectedSheet(Class<T> head, Integer sheetNo, String sheetName) {
        Map<Integer, ExcelSelectedResolve> selectedMap = resolveSelectedAnnotation(head);
        return EasyExcel.writerSheet(sheetNo, sheetName)
                .head(head)
                .registerWriteHandler(new SelectedSheetWriteHandler(selectedMap))
                .build();
    }

    /**
     * 不创建对象的写
     *
     * @param headList
     * @param dataList
     * @throws IOException
     */
    public static void exportExcel(HttpServletResponse response, String filename, String sheetName,
                                   List<List<String>> headList, List<List<String>> dataList) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ExcelWriter excelWriter = EasyExcel.write(out).registerWriteHandler(setHorizontalCellStyleStrategy()).head(headList).build();
        WriteSheet sheet = EasyExcel.writerSheet(sheetName).build();
        excelWriter.write(dataList, sheet);
        excelWriter.finish();
        byte[] content = out.toByteArray();
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename + ".xlsx", "UTF-8"));
        response.setContentType(MediaType.APPLICATION_OCTET_STREAM.toString());
        response.setHeader("Content-Length", "" + content.length);
        OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
        outputStream.write(content);
        outputStream.close();
    }

    /**
     * 带对象的写
     *
     * @param head
     * @param data
     * @param <T>
     * @throws IOException
     */
    public static <T> void exportExcel(HttpServletResponse response, String filename, String sheetName,
                                       Class<T> head, List<T> data) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ExcelWriter excelWriter = EasyExcel.write(out, head).registerWriteHandler(setHorizontalCellStyleStrategy()).build();
        WriteSheet sheet = EasyExcel.writerSheet(sheetName).build();
        excelWriter.write(data, sheet);
        excelWriter.finish();
        byte[] content = out.toByteArray();
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename + ".xlsx", "UTF-8"));
        response.setContentType(MediaType.APPLICATION_OCTET_STREAM.toString());
        response.setHeader("Content-Length", "" + content.length);
        OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
        outputStream.write(content);
        outputStream.close();
    }

    /**
     * 读excel，不限制条数
     *
     * @param file  文件
     * @param clazz 读取成的对象
     * @param <T>
     * @return
     * @throws IOException
     */
    public static <T> List<T> readExcel(MultipartFile file, Class<T> clazz) throws IOException {
        return readExcel(file, -1, clazz);
    }

    /**
     * 读取Excel（多个sheet可以用同一个实体类解析）
     * 读取成对象
     *
     * @param file       文件
     * @param limitLines 限制条数
     * @param clazz      读取成的对象
     * @param <T>
     * @return
     */
    public static <T> List<T> readExcel(MultipartFile file, int limitLines, Class<T> clazz) throws IOException {
        ExcelListener<T> excelListener = readExcelListener(file, limitLines, clazz);
        return Convert.toList(clazz, excelListener.getDataList());
    }

    /**
     * 解析导入excel数据  读取成map
     *
     * @param file      文件
     * @param lineLimit 限制条数
     * @return
     * @throws IOException
     */
    public static List<Map<Integer, String>> readExcel(MultipartFile file, Integer lineLimit) throws IOException {
        ExcelListener<Map<Integer, String>> excelListener = readExcelListener(file, lineLimit);
        return excelListener.getDataList();
    }

    /**
     * 读取excel对象 listener
     *
     * @param file       文件
     * @param limitLines 限制条数
     * @param clazz
     * @param <T>
     * @return
     * @throws IOException
     */
    public static <T> ExcelListener<T> readExcelListener(MultipartFile file, int limitLines, Class<T> clazz) throws IOException {
        ExcelListener<T> excelListener = new ExcelListener<T>(limitLines);
        return readExcelListener(file, clazz, excelListener);
    }

    /**
     * 读取excel map listener
     *
     * @param file      文件
     * @param lineLimit 限制条数
     * @return
     * @throws IOException
     */
    public static ExcelListener<Map<Integer, String>> readExcelListener(MultipartFile file, Integer lineLimit) throws IOException {
        ExcelListener<Map<Integer, String>> excelListener = new ExcelListener<>(lineLimit);
        return readExcelListener(file, null, excelListener);
    }

    /**
     * 获取excel数据并检查
     *
     * @param file       文件
     * @param head       头
     * @param headerName 标头名称
     * @param classifier 分类器
     * @return {@link List }<{@link T }>
     */
    public static <T> List<T> getExcelDataAndCheck(XUploadFile file, Class<T> head, String headerName, Function<? super T, ? extends String> classifier) {
        EasyExcelUtil.checkFileTypeIsExcel(file.getFileName());
        ExcelListener<T> excelListener = new ExcelListener<>(-1);
        EasyExcel.read(file.getInputStream(), head, excelListener).sheet().doRead();
        List<T> dataList = excelListener.getDataList();
        EasyExcelUtil.checkExcelDataSize(dataList);
        ValidationUtil.doValidate(dataList);
        //检查某个名称字段是否重复
        if (ObjectUtil.isNotNull(classifier) || CharSequenceUtil.isNotBlank(headerName)) {
            List<String> duplicate = dataList.stream()
                    .collect(Collectors.groupingBy(classifier, Collectors.counting()))
                    .entrySet()
                    .stream()
                    .filter(e -> e.getValue() > 1)
                    .map(Map.Entry::getKey)
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(duplicate)) {
                throw new XServiceException("表格内[" + headerName + "]列, 名称为[" + CollUtil.join(duplicate, ",") + "]重复");
            }
        }
        return dataList;
    }

    /*-------------------------------- private --------------------------------*/

    /**
     * 检查文件类型为excel
     *
     * @param fileName 文件名
     */
    private static void checkFileTypeIsExcel(String fileName) {
        // 截取文件后缀
        String fileSuffix = null;
        if (CharSequenceUtil.isNotBlank(fileName)) {
            fileSuffix = CharSequenceUtil.subAfter(fileName, ".", true);
        }
        String[] parts = {"xlsx", "xls"};
        // 判断文件是否是excel文件
        if (!CharSequenceUtil.equalsAny(fileSuffix, parts)) {
            throw new XServiceException(BusinessError.FileFormatError);
        }
    }

    /**
     * 检查excel数据大小
     *
     * @param dataList 数据列表
     */
    private static void checkExcelDataSize(List dataList) {
        if (CollUtil.isEmpty(dataList)) {
            throw new XServiceException(BusinessError.UndiscoveredData);
        }
    }

    private static <T> ExcelListener<T> readExcelListener(MultipartFile file, Class<T> clazz, ExcelListener<T> excelListener) throws IOException {
        ExcelReader excelReader = getReader(file.getInputStream(), file.getOriginalFilename(), clazz, excelListener);
        if (excelReader == null) {
            return excelListener;
        }
        List<ReadSheet> readSheetList = excelReader.excelExecutor().sheetList();
        for (ReadSheet readSheet : readSheetList) {
            excelReader.read(readSheet);
        }
        excelReader.finish();
        return excelListener;
    }

    private static <T> ExcelReader getReader(InputStream inputStream, String filename, Class<T> clazz, ExcelListener<T> excelListener) {
        try {
            if (filename == null || !filename.toLowerCase().endsWith(".xlsx")) {
                return null;
            }
            ExcelReader excelReader = clazz == null ? EasyExcel.read(inputStream, excelListener).build() :
                    EasyExcel.read(inputStream, clazz, excelListener).build();
            inputStream.close();
            return excelReader;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 解析表头类中的下拉注解
     *
     * @param head 表头类
     * @param <T>  泛型
     * @return Map<下拉框列索引, 下拉框内容> map
     */
    private static <T> Map<Integer, ExcelSelectedResolve> resolveSelectedAnnotation(Class<T> head) {
        Map<Integer, ExcelSelectedResolve> selectedMap = new HashMap<>(64);

        // getDeclaredFields(): 返回全部声明的属性；getFields(): 返回public类型的属性
        Field[] fields = head.getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            Field field = fields[i];
            // 解析注解信息
            ExcelSelected selected = field.getAnnotation(ExcelSelected.class);
            ExcelProperty property = field.getAnnotation(ExcelProperty.class);
            if (selected != null) {
                ExcelSelectedResolve excelSelectedResolve = new ExcelSelectedResolve();
                String[] source = excelSelectedResolve.resolveSelectedSource(selected);
                if (source != null && source.length > 0) {
                    excelSelectedResolve.setSource(source);
                    excelSelectedResolve.setFirstRow(selected.firstRow());
                    excelSelectedResolve.setLastRow(selected.lastRow());
                    if (property != null && property.index() >= 0) {
                        selectedMap.put(property.index(), excelSelectedResolve);
                    } else {
                        selectedMap.put(i, excelSelectedResolve);
                    }
                }
            }
        }
        return selectedMap;
    }

    /**
     * excel  样式处理
     *
     * @return
     */
    private static HorizontalCellStyleStrategy setHorizontalCellStyleStrategy() {
        /*******自定义列标题和内容的样式******/
        // 头的策略
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        // 背景设置
        headWriteCellStyle.setFillForegroundColor(IndexedColors.PALE_BLUE.getIndex());
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 10);
        headWriteCellStyle.setWriteFont(headWriteFont);
        // 内容的策略
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        // 背景绿色
        WriteFont contentWriteFont = new WriteFont();
        // 字体大小
        contentWriteFont.setFontHeightInPoints((short) 10);
        contentWriteCellStyle.setWriteFont(contentWriteFont);

        //设置 自动换行
        contentWriteCellStyle.setWrapped(true);
        //设置 垂直居中
        contentWriteCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        //设置 水平居中
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
        //设置边框样式
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }
}