package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.cloud.space.service.ISpaceInstitutionDetailCloudService;
import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendInput;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendOutput;
import pps.cloud.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadInput;
import pps.cloud.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadOutput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.*;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyAverageInput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendInput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.sys_organization.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.*;
import pps.core.prediction.enums.BusinessError;
import pps.core.prediction.mapper.*;
import pps.core.prediction.service.data.energy_consumption_analysis.*;
import pps.core.prediction.service.data.home_page.GetPowerAnalyseOutput;
import pps.core.prediction.service.data.third_current_well_condition.GetRealTimeInfoOutput;
import pps.core.prediction.service.data.third_current_well_condition.GetThirdCurrentWellConditionOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageData;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 能耗分析模块
 *
 * @author ZWT
 * @date 2023/11/23
 */
@XText("能耗分析模块")
@XService
public class EnergyConsumptionAnalysisService {

    /*------------------------------ 能耗分析 ------------------------------*/

    /**
     * 今日/昨日/同期电量统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link PowerStatisticsOutput }>
     */
    @XText("能耗分析--今日/昨日/同期电量统计")
    @XApiGet
    public XListResult<PowerStatisticsOutput> powerStatistics(XContext context, GetEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(this.getOrgIdsByPath(context, input.getOuId()));
        }
        List<PowerStatisticsOutput> outputs;
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, plantInput);
        //电站名称列表有值再查
        if (CollUtil.isNotEmpty(plantList)) {
            DateTime today = DateUtil.date();
            ThirdDailyAccumulationUpdateViewMapper mapper = context.getBean(ThirdDailyAccumulationUpdateViewMapper.class);
            List<ThirdDailyAccumulationUpdateView> viewList = mapper.selectPowerStatisticsByDate(ThirdDailyAccumulationUpdateView.builder()
                    .today(DateUtil.beginOfDay(today))
                    .yesterday(DateUtil.beginOfDay(DateUtil.yesterday()))
                    .lastYear(DateUtil.beginOfDay(DateUtil.offset(today, DateField.YEAR, -1)))
                    .stationNameList(plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()))
                    .build());
            outputs = XCopyUtils.copyNewList(viewList, PowerStatisticsOutput.class);
        } else {
            outputs = Collections.emptyList();
        }
        return XListResult.success(outputs);
    }

    /**
     * 用能分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerAnalyseOutput }>
     */
    @XText("能耗分析--用能分析")
    @XApiGet
    public XListResult<GetPowerAnalyseOutput> powerAnalyse(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLineWellheadInput wellheadInput = new DynamicQueryBasePowerLineWellheadInput();
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            wellheadInput.setLineName(stationName);
            plantInput.setLineName(stationName);
        } else {
            List<String> orgIds = this.getOrgIdsByPath(context, input.getOuId());
            wellheadInput.setOuIds(orgIds);
            plantInput.setOuIds(orgIds);
        }
        input.getBetweenDate();
        DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
        //用电功率
        Map<Date, BigDecimal> avgMap = this.queryPowerConsumptionHour(context,
                this.getPowerLineWellheadList(context, wellheadInput),
                input.getStartTime(),
                endTime
        );
        //发电功率
        Map<Date, BigDecimal> powerMap = this.queryGeneratedPowerHour(context,
                this.getPowerLinePlantViewList(context, plantInput),
                input.getStartTime(),
                endTime
        );
        //封装数据
        return XListResult.success(this.setPowerAnalyseList(avgMap, powerMap, input.getStartTime(), input.getEndTime()));
    }

    /**
     * 发电趋势
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerGenerationTrendOutput }>
     */
    @XText("能耗分析--发电趋势")
    @XApiGet
    public XListResult<GetPowerGenerationTrendOutput> powerGenerationTrend(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(this.getOrgIdsByPath(context, input.getOuId()));
        }
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, plantInput);
        Map<Date, BigDecimal> nowMap;
        Map<Date, BigDecimal> lastMap;
        input.getBetweenDate();
        //电站名称列表有值再查
        if (CollUtil.isNotEmpty(plantList)) {
            List<String> collect = plantList.stream()
                    .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                    .collect(Collectors.toList());
            //查本期
            DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
            nowMap = this.getPowerMap(context, input.getStartTime(), endTime, collect, 1, null, null);
            lastMap = this.getPowerMap(context, DateUtil.offset(input.getStartTime(), DateField.YEAR, -1), DateUtil.offset(endTime, DateField.YEAR, -1), collect, 1, null, null);
        } else {
            nowMap = Collections.emptyMap();
            lastMap = Collections.emptyMap();
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(input.getStartTime(), input.getEndTime(), DateField.DAY_OF_YEAR);
        List<GetPowerGenerationTrendOutput> outputs = new ArrayList<>(rangeToList.size());
        //封装数据
        BigDecimal powerGeneration;
        BigDecimal lastPowerGeneration;
        for (DateTime dateTime : rangeToList) {
            if (nowMap.containsKey(dateTime)) {
                powerGeneration = nowMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                powerGeneration = BigDecimal.ZERO;
            }
            if (lastMap.containsKey(DateUtil.offset(dateTime, DateField.YEAR, -1))) {
                lastPowerGeneration = lastMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                lastPowerGeneration = BigDecimal.ZERO;
            }
            outputs.add(GetPowerGenerationTrendOutput.builder()
                    .dateFormat(dateTime.toString())
                    .powerGeneration(powerGeneration)
                    .lastPowerGeneration(lastPowerGeneration)
                    .ratio(this.getAbsorptionRate(powerGeneration, lastPowerGeneration))
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 用电趋势
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetElectricityConsumptionTrendOutput }>
     */
    @XText("能耗分析--用电趋势")
    @XApiGet
    public XListResult<GetElectricityConsumptionTrendOutput> electricityConsumptionTrend(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(this.getOrgIdsByPath(context, input.getOuId()));
        }
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, plantInput);
        Map<Date, BigDecimal> nowMap;
        Map<Date, BigDecimal> lastMap;
        input.getBetweenDate();
        //电站名称列表有值再查
        if (CollUtil.isNotEmpty(plantList)) {
            List<String> collect = plantList.stream()
                    .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                    .collect(Collectors.toList());
            //查本期
            DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
            nowMap = this.getPowerMap(context, input.getStartTime(), endTime, collect, 2, null, null);
            lastMap = this.getPowerMap(context, DateUtil.offset(input.getStartTime(), DateField.YEAR, -1), DateUtil.offset(endTime, DateField.YEAR, -1), collect, 2, null, null);
        } else {
            nowMap = Collections.emptyMap();
            lastMap = Collections.emptyMap();
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(input.getStartTime(), input.getEndTime(), DateField.DAY_OF_YEAR);
        List<GetElectricityConsumptionTrendOutput> outputs = new ArrayList<>(rangeToList.size());
        //封装数据
        BigDecimal photovoltaicConsumption;
        BigDecimal lastPhotovoltaicConsumption;
        for (DateTime dateTime : rangeToList) {
            if (nowMap.containsKey(dateTime)) {
                photovoltaicConsumption = nowMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                photovoltaicConsumption = BigDecimal.ZERO;
            }
            if (lastMap.containsKey(DateUtil.offset(dateTime, DateField.YEAR, -1))) {
                lastPhotovoltaicConsumption = lastMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                lastPhotovoltaicConsumption = BigDecimal.ZERO;
            }
            outputs.add(GetElectricityConsumptionTrendOutput.builder()
                    .dateFormat(dateTime.toString())
                    .photovoltaicConsumption(photovoltaicConsumption)
                    .lastPhotovoltaicConsumption(lastPhotovoltaicConsumption)
                    .ratio(this.getAbsorptionRate(photovoltaicConsumption, lastPhotovoltaicConsumption))
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 用电分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetElectricityAnalysisOutput }>
     */
    @XText("能耗分析--用电分析")
    @XApiGet
    public XListResult<GetElectricityAnalysisOutput> electricityAnalysis(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(this.getOrgIdsByPath(context, input.getOuId()));
        }
        input.getBetweenDate();
        Map<Date, GetLineDailyElectricityTrendOutput> collect;
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, plantInput);
        if (CollUtil.isNotEmpty(plantList)) {
            DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
            collect = this.getDailyElectricityTrendMap(context, GetLineDailyElectricityTrendInput.builder()
                    .startTime(input.getStartTime())
                    .endTime(endTime)
                    .stationNameList(plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()))
                    .build());
        } else {
            collect = Collections.emptyMap();
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(input.getStartTime(), input.getEndTime(), DateField.DAY_OF_YEAR);
        List<GetElectricityAnalysisOutput> outputs = new ArrayList<>(rangeToList.size());
        //封装数据
        GetLineDailyElectricityTrendOutput getLineDailyElectricityTrendOutput;
        BigDecimal photovoltaicConsumption = BigDecimal.ZERO;
        BigDecimal municipalConsumption = BigDecimal.ZERO;
        BigDecimal lastPhotovoltaicConsumption = BigDecimal.ZERO;
        BigDecimal lastMunicipalConsumption = BigDecimal.ZERO;
        for (DateTime dateTime : rangeToList) {
            if (collect.containsKey(dateTime)) {
                getLineDailyElectricityTrendOutput = collect.get(dateTime);
                photovoltaicConsumption = getLineDailyElectricityTrendOutput.getPhotovoltaicPower();
                municipalConsumption = getLineDailyElectricityTrendOutput.getMunicipalPower();
                lastPhotovoltaicConsumption = getLineDailyElectricityTrendOutput.getLastPhotovoltaicPower();
                lastMunicipalConsumption = getLineDailyElectricityTrendOutput.getLastMunicipalPower();
            }
            outputs.add(GetElectricityAnalysisOutput.builder()
                    .dateFormat(dateTime.toString())
                    .photovoltaicConsumption(photovoltaicConsumption)
                    .municipalConsumption(municipalConsumption)
                    .lastPhotovoltaicConsumption(lastPhotovoltaicConsumption)
                    .lastMunicipalConsumption(lastMunicipalConsumption)
                    .ratio(this.getAbsorptionRate(photovoltaicConsumption, lastPhotovoltaicConsumption))
                    .build());
        }
        return XListResult.success(outputs);
    }

    /*------------------------------ 井场实时分析 ------------------------------*/

    /**
     * 发电详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetPowerGenerationDetailOutput }>
     */
    @XText("井场实时分析--发电详情")
    @XApiGet
    public XSingleResult<GetPowerGenerationDetailOutput> powerGenerationDetail(XContext context, GetEnergyConsumptionAnalysisInput input) {
        //光伏装机总量(kWh)
        BigDecimal photovoltaicPower = BigDecimal.ZERO;
        //发电功率(kWh)
        BigDecimal powerGeneration;
        //累计发电量(kWh)
        BigDecimal accumulatePowerGeneration;
        //就地消纳电量(KWh)
        BigDecimal inPlaceConsumption;
        //井场额定功率
        BigDecimal operatingLoad = BigDecimal.ZERO;
        //开井数量
        Integer openWellNumber;
        //日累计产液量(m³)
        BigDecimal dailyLiquidProduction;
        //日累计用电量(KW·h)
        BigDecimal dailyElectricityConsumption;
        //上网电量
        BigDecimal avoidPeakRate;
        //用电功率(kWh)
        BigDecimal powerConsumption;
        //绿电占比
        BigDecimal greenElectricityRate;
        List<String> orgIds = this.getOrgIdsByPath(context, input.getOuId());
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .ouIds(orgIds)
                .build());
        Set<String> plantSet = new HashSet<>(plantList.size());
        for (DynamicQueryBasePowerLinePlantViewOutput plant : plantList) {
            photovoltaicPower.add(plant.getTotalPower());
            plantSet.add(plant.getStationName());
        }
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        //累计发电
        ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
        ThirdDailyAccumulationUpdateEnt analysisEnt = analysisMapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                        "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                        "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                        "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption")
                .lambda()
                .eq(ThirdDailyAccumulationUpdateEnt::getSaveDate, today)
                .in(ThirdDailyAccumulationUpdateEnt::getStationName, plantSet)
        );
        accumulatePowerGeneration = analysisEnt.getPhotovoltaicPower();
        inPlaceConsumption = analysisEnt.getInPlaceConsumption();
        dailyLiquidProduction = analysisEnt.getDailyLiquidProduction();
        dailyElectricityConsumption = analysisEnt.getDailyElectricityConsumption();
        greenElectricityRate = this.getAbsorptionRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption());
        avoidPeakRate = BaseUtils.divide(analysisEnt.getPhotovoltaicPower(), analysisEnt.getInPlaceConsumption());
        //发电功率
        ThirdActivePowerDailyUpdateMapper activeMapper = context.getBean(ThirdActivePowerDailyUpdateMapper.class);
        ThirdActivePowerDailyUpdateEnt activeEnt = activeMapper.selectOne(new QueryWrapper<ThirdActivePowerDailyUpdateEnt>()
                .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power")
                .lambda()
                .eq(ThirdActivePowerDailyUpdateEnt::getSaveDate, today)
                .in(ThirdActivePowerDailyUpdateEnt::getStationName, plantSet)
        );
        powerGeneration = activeEnt.getPhotovoltaicPower();
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = this.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .ouIds(orgIds)
                .build());
        Set<String> wellSet = new HashSet<>(wellheadList.size());
        for (DynamicQueryBasePowerLineWellheadViewOutput well : wellheadList) {
            operatingLoad.add(well.getRatedPower());
            wellSet.add(well.getWellNumber());
        }
        //用电功率
        ThirdWellAvgActivePowerMapper avgMapper = context.getBean(ThirdWellAvgActivePowerMapper.class);
        ThirdWellAvgActivePowerEnt activePowerEnt = avgMapper.selectOne(new QueryWrapper<ThirdWellAvgActivePowerEnt>()
                .select("IFNULL( SUM( avg_active_power ), 0 ) AS avg_active_power")
                .lambda()
                .in(ThirdWellAvgActivePowerEnt::getWellNumber, wellSet)
                .apply("DATE(input_time) = {0}", today)
        );
        powerConsumption = activePowerEnt.getAvgActivePower();
        //开井数量
        ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
        List<ThirdCurrentWellConditionEnt> list = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                .in(ThirdCurrentWellConditionEnt::getWellNumber, wellSet)
        );
        openWellNumber = (int) list.stream()
                .filter(l -> CharSequenceUtil.equals(l.getWellStatus(), "开井"))
                .count();
        return XSingleResult.success(GetPowerGenerationDetailOutput.builder()
                .photovoltaicPower(photovoltaicPower)
                .powerGeneration(powerGeneration)
                .accumulatePowerGeneration(accumulatePowerGeneration)
                .inPlaceConsumption(inPlaceConsumption)
                .operatingLoad(operatingLoad)
                .openWellNumber(openWellNumber)
                .dailyLiquidProduction(dailyLiquidProduction)
                .dailyElectricityConsumption(dailyElectricityConsumption)
                .avoidPeakRate(avoidPeakRate)
                .powerConsumption(powerConsumption)
                .greenElectricityRate(greenElectricityRate)
                .build());
    }

    /**
     * 井场实时监控
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWellFieldSituationOutput }>
     */
    @XText("井场实时分析--井场实时监控")
    @XApiGet
    public XSingleResult<GetWellFieldSituationOutput> wellFieldSituation(XContext context, GetEnergyConsumptionAnalysisInput input) {
        int sumWellNumber = 0;
        int normallyOpenWellNumber = 0;
        int interOpenWellNumber = 0;
        int openWellNumber = 0;
        int stopWellNumber = 0;
        List<GetThirdCurrentWellConditionOutput> currentList = Collections.emptyList();
        GetWellFieldSituationOutput output = new GetWellFieldSituationOutput();
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellListByOuId(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isNotEmpty(wellList)) {
            sumWellNumber = wellList.size();
            Set<String> set = new HashSet<>(wellList.size());
            for (DynamicQueryBaseWellheadOutput out : wellList) {
                set.add(out.getWellNumber());
                if (CharSequenceUtil.equals(out.getRunTypeKey(), BusinessConstant.INTERVAL_PUMPING_WELL)) {
                    interOpenWellNumber++;
                } else {
                    normallyOpenWellNumber++;
                }
            }
            ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
            List<ThirdCurrentWellConditionEnt> list = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                    .in(ThirdCurrentWellConditionEnt::getWellNumber, set)
            );
            if (CollUtil.isNotEmpty(list)) {
                //查生效中的所有井口
                ISpaceInstitutionDetailCloudService service = context.getBean(ISpaceInstitutionDetailCloudService.class);
                XListResult<GetSpaceInstitutionWellheadOutput> result = service.getInstitutionWellheadList(context, new GetSpaceInstitutionWellheadInput());
                result.throwIfFail();
                List<GetSpaceInstitutionWellheadOutput> wellheadOutputList = result.getResult();
                Map<String, BigDecimal> collect;
                if (CollUtil.isNotEmpty(wellheadOutputList)) {
                    collect = wellheadOutputList.stream()
                            .collect(Collectors.toMap(GetSpaceInstitutionWellheadOutput::getWellNumber,
                                    GetSpaceInstitutionWellheadOutput::getRunDuration));
                } else {
                    collect = Collections.emptyMap();
                }
                BigDecimal surplusTime;
                currentList = new ArrayList<>(list.size());
                for (ThirdCurrentWellConditionEnt ent : list) {
                    if (CharSequenceUtil.equals(ent.getWellStatus(), "开井")) {
                        openWellNumber++;
                    } else {
                        stopWellNumber++;
                    }
                    Date nextTime = ent.getNextTime();
                    if (ObjectUtil.isNotNull(nextTime) && CharSequenceUtil.contains(DateUtil.date(ent.getNextTime()).toString(), ThirdPartyApiConstant.DATE_FLAG)) {
                        nextTime = null;
                    }
                    if (collect.containsKey(ent.getWellNumber())) {
                        surplusTime = collect.get(ent.getWellNumber()).subtract(ent.getRunningTime());
                    } else {
                        surplusTime = BigDecimal.ZERO;
                    }
                    currentList.add(
                            GetThirdCurrentWellConditionOutput.builder()
                                    .wellNumber(ent.getWellNumber())
                                    .nextTime(nextTime)
                                    .wellStatus(ent.getWellStatus())
                                    .cumulativeProduction(ent.getCumulativeProduction())
                                    .runningTime(ent.getRunningTime())
                                    .surplusTime(surplusTime)
                                    .build()
                    );
                }
            }
        }
        output.setList(currentList);
        output.setSumWellNumber(sumWellNumber);
        output.setNormallyOpenWellNumber(normallyOpenWellNumber);
        output.setInterOpenWellNumber(interOpenWellNumber);
        output.setOpenWellNumber(openWellNumber);
        output.setStopWellNumber(stopWellNumber);
        return XSingleResult.success(output);
    }

    /**
     * 本日用电对比
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerAnalyseOutput }>
     */
    @XText("井场实时分析--本日用电对比")
    @XApiGet
    public XListResult<GetPowerAnalyseOutput> electricityConsumptionContrast(XContext context, GetEnergyConsumptionAnalysisInput input) {
        List<String> orgIds = this.getOrgIdsByPath(context, input.getOuId());
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        DateTime tomorrow = DateUtil.beginOfDay(DateUtil.tomorrow());
        //用电功率
        Map<Date, BigDecimal> avgMap = this.queryPowerConsumptionHour(context,
                this.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                        .ouIds(orgIds)
                        .build()),
                today,
                tomorrow
        );
        //发电功率
        Map<Date, BigDecimal> powerMap = this.queryGeneratedPowerHour(context,
                this.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                        .ouIds(orgIds)
                        .build()),
                today,
                tomorrow
        );
        //封装数据
        return XListResult.success(this.setPowerAnalyseList(avgMap, powerMap, today, tomorrow));
    }

    /**
     * 本日预测/实际功率对比
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPredictedPowerContrastOutput }>
     */
    @XText("井场实时分析--本日预测/实际功率对比")
    @XApiGet
    public XListResult<GetPredictedPowerContrastOutput> predictedPowerContrast(XContext context, GetEnergyConsumptionAnalysisInput input) {
        Map<String, BigDecimal> shortMap;
        Map<Integer, BigDecimal> powerMap;
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .ouIds(this.getOrgIdsByPath(context, input.getOuId()))
                .build());
        if (CollUtil.isNotEmpty(plantList)) {
            List<String> plantIds = plantList.stream()
                    .map(DynamicQueryBasePowerLinePlantViewOutput::getPlantId)
                    .collect(Collectors.toList());
            //预测发电量
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            shortMap = mapper.selectShortPowerHourList(PredictedPowerView.builder()
                            .createTime(today)
                            .plantIds(plantIds)
                            .build()).stream()
                    .collect(Collectors.toMap(PredictedPowerView::getHourTime, PredictedPowerView::getPower));
            //实际功率
            ThirdActivePowerDailyUpdateViewMapper nowMapper = context.getBean(ThirdActivePowerDailyUpdateViewMapper.class);
            powerMap = nowMapper.selectEnergyUseCurve(ThirdActivePowerDailyUpdateView.builder()
                            .stationIds(plantIds)
                            .saveDate(today)
                            .build()).stream()
                    .collect(Collectors.toMap(ThirdActivePowerDailyUpdateView::getHourNumber, ThirdActivePowerDailyUpdateView::getPhotovoltaicPower));
        } else {
            shortMap = Collections.emptyMap();
            powerMap = Collections.emptyMap();
        }
        //封装数据
        List<GetPredictedPowerContrastOutput> outputs = new ArrayList<>(24);
        BigDecimal predictPower;
        BigDecimal actualPower;
        for (int i = 0; i < 24; i++) {
            String value = String.valueOf(i);
            predictPower = shortMap.getOrDefault(value, BigDecimal.ZERO);
            actualPower = powerMap.getOrDefault(value, BigDecimal.ZERO);
            outputs.add(
                    GetPredictedPowerContrastOutput.builder()
                            .hourNumber(i)
                            .predictPower(predictPower)
                            .actualPower(actualPower)
                            .build()
            );
        }
        return XListResult.success(outputs);
    }

    /*------------------------------ 间开效果评价 ------------------------------*/

    /**
     * 月度总览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetMonthlyOverviewOutput }>
     */
    @XText("间开效果评价--月度总览")
    @XApiGet
    public XSingleResult<GetMonthlyOverviewOutput> monthlyOverview(XContext context, GetEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(this.getOrgIdsByPath(context, input.getOuId()));
        }
        BigDecimal accumulatePowerGeneration = BigDecimal.ZERO;
        BigDecimal dailyElectricityConsumption = BigDecimal.ZERO;
        BigDecimal greenElectricityRate = BigDecimal.ZERO;
        BigDecimal economicBenefit = BigDecimal.ZERO;
        BigDecimal carbonReduction = BigDecimal.ZERO;
        DateTime end = DateUtil.beginOfDay(DateUtil.date());
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, plantInput);
        if (CollUtil.isNotEmpty(plantList)) {
            DateTime start = DateUtil.beginOfMonth(end);
            //累计发电
            ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            ThirdDailyAccumulationUpdateEnt analysisEnt = analysisMapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                            "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                            "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                            "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption")
                    .lambda()
                    .between(ThirdDailyAccumulationUpdateEnt::getSaveDate, start, end)
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()))
            );
            accumulatePowerGeneration = analysisEnt.getPhotovoltaicPower();
            dailyElectricityConsumption = analysisEnt.getDailyElectricityConsumption();
            greenElectricityRate = this.getAbsorptionRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption());
            economicBenefit = this.calculateEconomicBenefit(analysisEnt.getPhotovoltaicPower());
            carbonReduction = this.calculateCarbonReduction(analysisEnt.getPhotovoltaicPower());
        }
        ISpaceOptimizeShortCloudService service = context.getBean(ISpaceOptimizeShortCloudService.class);
        XSingleResult<GetMonthlyWellCountOutput> result = service.monthlyWellCount(context, GetMonthlyWellCountInput.builder()
                .month(end.monthBaseOne())
                .build());
        result.throwIfFail();
        Long wellNumber = result.getResult().getWellNumber();
        return XSingleResult.success(GetMonthlyOverviewOutput.builder()
                .accumulatePowerGeneration(accumulatePowerGeneration)
                .dailyElectricityConsumption(dailyElectricityConsumption)
                .greenElectricityRate(greenElectricityRate)
                .wellNumber(wellNumber)
                .economicBenefit(economicBenefit)
                .carbonReduction(carbonReduction)
                .build());
    }

    /**
     * 本月累计节电经济效益/减碳量
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetEconomicBenefitsOutput }>
     */
    @XText("间开效果评价--本月累计节电经济效益/减碳量")
    @XApiGet
    public XListResult<GetEconomicBenefitsOutput> economicBenefitCarbonReduction(XContext context, GetEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(this.getOrgIdsByPath(context, input.getOuId()));
        }
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, plantInput);
        DateTime end = DateUtil.beginOfDay(DateUtil.yesterday());
        DateTime start;
        Map<Date, BigDecimal> map;
        //区分环境
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        List<DateTime> rangeToList;
        Integer dateType = ObjectUtil.defaultIfNull(input.getDateType(), 0);
        if (dateType.equals(1)) {
            start = DateUtil.beginOfYear(end);
            rangeToList = DateUtil.rangeToList(start, end, DateField.MONTH);
        } else {
            start = DateUtil.beginOfMonth(end);
            rangeToList = DateUtil.rangeToList(start, end, DateField.DAY_OF_YEAR);
        }
        if (CollUtil.isNotEmpty(plantList)) {
            //实际发电量
            map = this.getPowerMap(context, start, DateUtil.offsetDay(end, 1),
                    plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()), 1, oilFieldCode, dateType);
        } else {
            map = Collections.emptyMap();
        }
        //封装数据
        List<GetEconomicBenefitsOutput> outputs = new ArrayList<>(rangeToList.size());
        BigDecimal economicBenefit;
        BigDecimal carbonReduction;
        BigDecimal electricityFees;
        BigDecimal saveElectricity;
        switch (oilFieldCode) {
            case BusinessConstant.ENV_CQ:
                for (DateTime dateTime : rangeToList) {
                    if (map.containsKey(dateTime)) {
                        economicBenefit = this.calculateEconomicBenefit(map.get(dateTime));
                        carbonReduction = this.calculateCarbonReduction(map.get(dateTime));
                    } else {
                        economicBenefit = BigDecimal.ZERO;
                        carbonReduction = BigDecimal.ZERO;
                    }
                    outputs.add(GetEconomicBenefitsOutput.builder()
                            .dateFormat(dateTime)
                            .economicBenefit(economicBenefit)
                            .carbonReduction(carbonReduction)
                            .build());
                }
                break;
            case BusinessConstant.ENV_SY:
                for (DateTime dateTime : rangeToList) {
                    if (map.containsKey(dateTime)) {
                        saveElectricity = map.get(dateTime);
                        carbonReduction = this.calculateCarbonReduction(map.get(dateTime));
                    } else {
                        saveElectricity = BigDecimal.ZERO;
                        carbonReduction = BigDecimal.ZERO;
                    }
                    outputs.add(GetEconomicBenefitsOutput.builder()
                            .dateFormat(dateTime)
                            .carbonReduction(carbonReduction)
                            .saveElectricity(saveElectricity)
                            .electricityFees(BigDecimal.ZERO)
                            .build());
                }
                break;
            default:
        }
        return XListResult.success(outputs);
    }

    /**
     * 井口分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult }<{@link GetWellheadAnalysisOutput }>
     */
    @XText("间开效果评价--井口分析")
    @XApiGet
    public XPageResult<GetWellheadAnalysisOutput> wellheadAnalysis(XContext context, GetWellheadAnalysisInput input) {
        DynamicQueryBasePowerLineWellheadInput wellheadInput = new DynamicQueryBasePowerLineWellheadInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            wellheadInput.setLineName(stationName);
        } else {
            List<String> orgIds = this.getOrgIdsByPath(context, input.getOuId());
            wellheadInput.setOuIds(orgIds);
        }
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = this.getPowerLineWellheadList(context, wellheadInput);
        if (CollUtil.isNotEmpty(wellheadList)) {
            DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
            IDailyElectricityTrendCloudService service = context.getBean(IDailyElectricityTrendCloudService.class);
            GetWellDailyElectricityTrendInput build = GetWellDailyElectricityTrendInput.builder()
                    .createDate(yesterday)
                    .wellNumberList(wellheadList.stream().map(DynamicQueryBasePowerLineWellheadViewOutput::getWellNumber)
                            .collect(Collectors.toList()))
                    .wellNumber(input.getWellNumber())
                    .build();
            build.setPage(input.getPage());
            build.setLimit(input.getLimit());
            XPageResult<GetWellDailyElectricityTrendOutput> result = service.queryWellheadAnalysisPage(context, build);
            result.throwIfFail();
            XPageData<GetWellDailyElectricityTrendOutput> pageInfo = result.getResult();
            List<GetWellDailyElectricityTrendOutput> list = pageInfo.getItems();
            if (CollUtil.isNotEmpty(list)) {
                return XPageResult.success(XCopyUtils.copyNewList(list, GetWellheadAnalysisOutput.class), input, pageInfo.getTotal());
            }
        }
        return XPageResult.success(Collections.emptyList(), input, 0);
    }

    /*------------------------------ 井组监控 ------------------------------*/

    /**
     * 井组生产情况
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWellProductionStatusOutput }>
     */
    @XText("井组监控--井组生产情况")
    @XApiGet
    public XSingleResult<GetWellProductionStatusOutput> wellProductionStatus(XContext context, GetEnergyConsumptionAnalysisInput input) {
        int sumWellNumber = 0;
        int normallyOpenWellNumber = 0;
        int interOpenWellNumber = 0;
        BigDecimal liquidProduction = BigDecimal.ZERO;
        BigDecimal electricityConsumption = BigDecimal.ZERO;
        BigDecimal carbonReduction = BigDecimal.ZERO;
        BigDecimal economicBenefit = BigDecimal.ZERO;
        List<String> orgIdsByPath = this.getOrgIdsByPath(context, input.getOuId());
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellListByOuId(context, orgIdsByPath);
        if (CollUtil.isNotEmpty(wellList)) {
            sumWellNumber = wellList.size();
            List<String> set = new ArrayList<>(wellList.size());
            for (DynamicQueryBaseWellheadOutput out : wellList) {
                set.add(out.getWellNumber());
                if (CharSequenceUtil.equals(out.getRunTypeKey(), BusinessConstant.INTERVAL_PUMPING_WELL)) {
                    interOpenWellNumber++;
                } else {
                    normallyOpenWellNumber++;
                }
            }
            //日均产液量
            IDailyElectricityTrendCloudService service = context.getBean(IDailyElectricityTrendCloudService.class);
            XSingleResult<GetWellDailyElectricityTrendOutput> wellDailyAverageResult = service.queryWellDailyAverage(context, GetWellDailyAverageInput.builder()
                    .systemSource(oilFieldCode)
                    .wellNumberList(set)
                    .build());
            wellDailyAverageResult.throwIfFail();
            GetWellDailyElectricityTrendOutput result = wellDailyAverageResult.getResult();
            liquidProduction = result.getDailyLiquidProduction();
            electricityConsumption = result.getDailyElectricityConsumption();
        }
        //累计数据
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = this.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .ouIds(orgIdsByPath)
                .build());
        if (CollUtil.isNotEmpty(plantList)) {
            ThirdDailyAccumulationUpdateMapper mapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            ThirdDailyAccumulationUpdateEnt analysisEnt = mapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power")
                    .lambda()
                    .eq(ThirdDailyAccumulationUpdateEnt::getSystemSource, oilFieldCode)
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()))
            );
            economicBenefit = this.calculateEconomicBenefit(analysisEnt.getPhotovoltaicPower());
            carbonReduction = this.calculateCarbonReduction(analysisEnt.getPhotovoltaicPower());
        }
        return XSingleResult.success(GetWellProductionStatusOutput.builder()
                .sumWellNumber(sumWellNumber)
                .normallyOpenWellNumber(normallyOpenWellNumber)
                .interOpenWellNumber(interOpenWellNumber)
                .liquidProduction(liquidProduction)
                .electricityConsumption(electricityConsumption)
                .carbonReduction(carbonReduction)
                .economicBenefit(economicBenefit)
                .build());
    }

    /**
     * 井口实时信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetRealTimeInfoOutput }>
     */
    @XText("井组监控--井口实时信息")
    @XApiGet
    public XListResult<GetRealTimeInfoOutput> wellRealTimeInfo(XContext context, GetEnergyConsumptionAnalysisInput input) {
        List<GetRealTimeInfoOutput> currentList = Collections.emptyList();
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellListByOuId(context, this.getOrgIdsByPath(context, input.getOuId()), input.getRunTypeKey());
        if (CollUtil.isNotEmpty(wellList)) {
            String wellStatus = input.getWellStatus();
            Map<String, DynamicQueryBaseWellheadOutput> collect = new HashMap<>(wellList.size());
            List<String> wellIds = new ArrayList<>(wellList.size());
            for (DynamicQueryBaseWellheadOutput well : wellList) {
                collect.put(well.getWellNumber(), well);
                wellIds.add(well.getId());
            }
            ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
            List<ThirdCurrentWellConditionEnt> list = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                    .in(ThirdCurrentWellConditionEnt::getWellNumber, collect.keySet())
                    .eq(CharSequenceUtil.isNotBlank(wellStatus), ThirdCurrentWellConditionEnt::getWellStatus, wellStatus)
            );
            if (CollUtil.isNotEmpty(list)) {
                //当日间开
                ISpaceOptimizeShortCloudService service = context.getBean(ISpaceOptimizeShortCloudService.class);
                XListResult<GetSpaceOptimizeShortWellheadOutput> result = service.dailyOptimizationResult(context, GetSpaceOptimizeShortWellheadInput.builder()
                        .wellheadIds(wellIds)
                        .build());
                result.throwIfFail();
                List<GetSpaceOptimizeShortWellheadOutput> optimizeWellheadList = result.getResult();
                Map<String, List<SpaceOptimizeDateDuration>> listMap;
                if (CollUtil.isNotEmpty(optimizeWellheadList)) {
                    listMap = optimizeWellheadList.stream()
                            .collect(Collectors.toMap(GetSpaceOptimizeShortWellheadOutput::getWellNumber, GetSpaceOptimizeShortWellheadOutput::getOptimizeList));
                } else {
                    listMap = Collections.emptyMap();
                }
                //查生效中的所有井口
                BigDecimal nextOpen;
                currentList = new ArrayList<>(list.size());
                DateTime now = DateUtil.date();
                for (ThirdCurrentWellConditionEnt ent : list) {
                    if (ObjectUtil.isNull(ent.getNextTime()) ||
                            CharSequenceUtil.contains(DateUtil.date(ent.getNextTime()).toString(), ThirdPartyApiConstant.DATE_FLAG) ||
                            DateUtil.compare(now, ent.getNextTime()) >= 0) {
                        nextOpen = BigDecimal.valueOf(-1);
                    } else {
                        nextOpen = BaseUtils.divide(BigDecimal.valueOf(DateUtil.between(ent.getNextTime(), now, DateUnit.MINUTE)), BusinessConstant.BIG_DECIMAL_60);
                    }

                    currentList.add(
                            GetRealTimeInfoOutput.builder()
                                    .wellNumber(ent.getWellNumber())
                                    .nextOpen(nextOpen)
                                    .wellStatus(ent.getWellStatus())
                                    .runTypeName(collect.get(ent.getWellNumber()).getRunTypeName())
                                    .ratedPower(collect.get(ent.getWellNumber()).getRatedPower())
                                    .optimizeList(listMap.getOrDefault(ent.getWellNumber(), Collections.emptyList()))
                                    .build()
                    );
                }
            }
        }
        return XListResult.success(currentList);
    }

    /*------------------------------ 能耗分析(废弃) ------------------------------*/

    /**
     * 能耗分析--能耗概览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetEnergyConsumptionAnalysisViewOutput}>
     */
    @Deprecated
    @XText("能耗分析--能耗概览")
    @XApiGet
    public XSingleResult<GetEnergyConsumptionAnalysisViewOutput> getEnergyConsumptionAnalysisView(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        GetEnergyConsumptionAnalysisViewOutput output = new GetEnergyConsumptionAnalysisViewOutput();
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isEmpty(lineList)) {
            output.setPowerGeneration(BigDecimal.ZERO);
            output.setPowerConsumption(BigDecimal.ZERO);
            output.setAbsorptionRate(BigDecimal.ZERO);
        } else {
            List<String> lineNameList = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toList());
            input.getBetweenDate();
            ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            ThirdDailyAccumulationUpdateEnt analysisEnt = analysisMapper.selectOne(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                            "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                            "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                            "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption")
                    .lambda()
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, lineNameList)
                    .between(ThirdDailyAccumulationUpdateEnt::getCreateDate, input.getStartTime(), input.getEndTime())
            );
            output.setPowerGeneration(analysisEnt.getPhotovoltaicPower().setScale(2, RoundingMode.HALF_UP));
            output.setPowerConsumption(analysisEnt.getDailyElectricityConsumption().setScale(2, RoundingMode.HALF_UP));
            output.setAbsorptionRate(this.getAbsorptionRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption()));
        }
        return XSingleResult.success(output);
    }

    /**
     * 能耗分析--绿电消纳/用电趋势
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryEnergyConsumptionAnalysisViewOutput}>
     */
    @Deprecated
    @XText("能耗分析--绿电消纳/用电趋势")
    @XApiGet
    public XListResult<DynamicQueryEnergyConsumptionAnalysisViewOutput> queryEnergyConsumptionAnalysisList(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        List<DynamicQueryEnergyConsumptionAnalysisViewOutput> list = new ArrayList<>(16);
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isNotEmpty(lineList)) {
            List<String> lineNameList = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toList());
            input.getBetweenDate();
            ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            List<ThirdDailyAccumulationUpdateEnt> analysisEntList = analysisMapper.selectList(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                            "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                            "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                            "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption",
                            "DATE( create_date ) AS create_date")
                    .lambda()
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, lineNameList)
                    .between(ThirdDailyAccumulationUpdateEnt::getCreateDate, input.getStartTime(), input.getEndTime())
                    .last("GROUP BY DATE( create_date ) ")
            );
            if (CollUtil.isNotEmpty(analysisEntList)) {
                for (ThirdDailyAccumulationUpdateEnt ent : analysisEntList) {
                    list.add(DynamicQueryEnergyConsumptionAnalysisViewOutput.builder()
                            .powerConsumption(ent.getDailyElectricityConsumption())
                            .powerGeneration(ent.getPhotovoltaicPower())
                            .dataDate(ent.getCreateDate())
                            .dateFormat(DateUtil.format(ent.getCreateDate(), "MM-dd"))
                            .absorptionRate(this.getAbsorptionRate(ent.getInPlaceConsumption(), ent.getDailyElectricityConsumption()))
                            .build());
                }
            }
        }
        return XListResult.success(list);
    }

    /**
     * 能耗分析--用电详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryEnergyConsumptionAnalysisViewOutput}>
     */
    @Deprecated
    @XText("能耗分析--用电详情")
    @XApiGet
    public XPageResult<QueryEnergyConsumptionAnalysisViewOutput> queryEnergyConsumptionAnalysisView(XContext context, QueryEnergyConsumptionAnalysisPageInput input) {
        List<QueryEnergyConsumptionAnalysisViewOutput> outputs = new ArrayList<>(input.getLimit());
        List<ThirdDailyAccumulationUpdateEnt> list;
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isNotEmpty(lineList)) {
            List<String> lineNameList = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toList());
            PageHelper.startPage(input.getPage(), input.getLimit());
            input.getBetweenDate();
            ThirdDailyAccumulationUpdateMapper mapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            Date startTime = input.getStartTime();
            Date endTime = input.getEndTime();
            //查本期数据
            list = mapper.selectList(this.getQueryWrapper4Page(lineNameList, startTime, endTime));
            if (CollUtil.isNotEmpty(list)) {
                PageHelper.clearPage();
                //查上期数据
                switch (input.getDateType()) {
                    case 1:
                        endTime = DateUtil.offsetDay(startTime, -1);
                        startTime = DateUtil.offsetDay(startTime, -7);
                        break;
                    case 2:
                        endTime = DateUtil.offsetDay(startTime, -1);
                        startTime = DateUtil.offsetDay(startTime, -30);
                        break;
                    case 3:
                        int betweenDay = (int) DateUtil.betweenDay(startTime, endTime, true) + 1;
                        endTime = DateUtil.offsetDay(startTime, -1);
                        startTime = DateUtil.offsetDay(startTime, -betweenDay);
                        break;
                    default:
                        throw new XServiceException(BusinessError.DateTypeError);
                }
                List<ThirdDailyAccumulationUpdateEnt> periodList = mapper.selectList(this.getQueryWrapper4Page(lineNameList, startTime, endTime));
                if (CollUtil.isNotEmpty(periodList)) {
                    for (ThirdDailyAccumulationUpdateEnt ent : list) {
                        EnergyConsumptionAnalysisEnt period;
                        ThirdDailyAccumulationUpdateEnt one = CollUtil.findOne(periodList, x -> CharSequenceUtil.equals(ent.getStationName(), x.getStationName()));
                        if (Objects.isNull(one)) {
                            period = EnergyConsumptionAnalysisEnt.builder()
                                    .powerConsumption(BigDecimal.ZERO)
                                    .powerGeneration(BigDecimal.ZERO)
                                    .build();
                        } else {
                            period = EnergyConsumptionAnalysisEnt.builder()
                                    .powerConsumption(one.getDailyElectricityConsumption())
                                    .powerGeneration(one.getPhotovoltaicPower())
                                    .build();
                        }
                        outputs.add(QueryEnergyConsumptionAnalysisViewOutput.builder()
                                .lineName(ent.getStationName())
                                .powerGeneration(ent.getPhotovoltaicPower())
                                .generationCycleRatio(this.getCycleRatio(ent.getPhotovoltaicPower(), period.getPowerGeneration()))
                                .powerConsumption(ent.getDailyElectricityConsumption())
                                .consumptionCycleRatio(this.getCycleRatio(ent.getDailyElectricityConsumption(), period.getPowerConsumption()))
                                .absorptionRate(this.getAbsorptionRate(ent.getInPlaceConsumption(), ent.getDailyElectricityConsumption()))
                                .build());
                    }
                } else {
                    for (ThirdDailyAccumulationUpdateEnt ent : list) {
                        outputs.add(QueryEnergyConsumptionAnalysisViewOutput.builder()
                                .lineName(ent.getStationName())
                                .powerGeneration(ent.getPhotovoltaicPower())
                                .generationCycleRatio(BigDecimal.ZERO)
                                .powerConsumption(ent.getDailyElectricityConsumption())
                                .consumptionCycleRatio(BigDecimal.ZERO)
                                .absorptionRate(this.getAbsorptionRate(ent.getInPlaceConsumption(), ent.getDailyElectricityConsumption()))
                                .build());
                    }
                }
            }
        } else {
            list = Collections.emptyList();
        }
        PageInfo<ThirdDailyAccumulationUpdateEnt> pageInfo = new PageInfo<>(list);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 分页功能封装查询条件
     *
     * @param lineNames 线路名称
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return {@link LambdaQueryWrapper}<{@link EnergyConsumptionAnalysisEnt}>
     */
    private LambdaQueryWrapper<ThirdDailyAccumulationUpdateEnt> getQueryWrapper4Page(Collection<String> lineNames, Date startTime, Date endTime) {
        return new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                        "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                        "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                        "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption",
                        "station_name")
                .lambda()
                .in(ThirdDailyAccumulationUpdateEnt::getStationName, lineNames)
                .between(ThirdDailyAccumulationUpdateEnt::getCreateDate, startTime, endTime)
                .groupBy(ThirdDailyAccumulationUpdateEnt::getStationName)
                .orderByAsc(ThirdDailyAccumulationUpdateEnt::getPhotovoltaicPower);
    }

    /**
     * 获取消纳率
     * 消纳率:就地消纳量/累计用电*100%
     *
     * @param powerGeneration  发电
     * @param powerConsumption 功耗
     * @return {@link BigDecimal}
     */
    private BigDecimal getAbsorptionRate(BigDecimal powerGeneration, BigDecimal powerConsumption) {
        BigDecimal absorptionRate;
        if (powerConsumption.compareTo(BigDecimal.ZERO) > 0) {
            absorptionRate = powerGeneration.divide(powerConsumption, 4, RoundingMode.HALF_UP)
                    .multiply(BusinessConstant.BIG_DECIMAL_100);
        } else {
            absorptionRate = BigDecimal.ZERO;
        }
        return absorptionRate;
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    private List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 查询组织列表下所有线路
     *
     * @param context  上下文
     * @param ouIdList ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, List<String> ouIdList) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineListByOuIdList(context, DynamicQueryBasePowerLineInput.builder()
                .ouIdList(ouIdList)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }

    /**
     * 获取输电线井口列表
     *
     * @param context 上下文
     * @param input   入参
     * @return {@link List}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    private List<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = service.getPowerLineWellheadList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取输电线电站列表
     *
     * @param context 上下文
     * @param input   入参
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantViewOutput}>
     */
    private List<DynamicQueryBasePowerLinePlantViewOutput> getPowerLinePlantViewList(XContext context, DynamicQueryBasePowerLinePlantInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantViewOutput> result = service.getPowerLinePlantViewList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 电量统计
     *
     * @param context         上下文
     * @param startTime       开始时间
     * @param endTime         结束时间
     * @param stationNameList 车站名称列表
     * @param flag            旗帜 (1_发电,2_用电)
     * @param oilFieldCode    部署环境
     * @param dateType        日期范围类型(0_本月;1_本年)
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    private Map<Date, BigDecimal> getPowerMap(XContext context, Date startTime, Date endTime, List<String> stationNameList, int flag, String oilFieldCode, Integer dateType) {
        ThirdDailyAccumulationUpdateViewMapper mapper = context.getBean(ThirdDailyAccumulationUpdateViewMapper.class);
        List<ThirdDailyAccumulationUpdateView> powerList = mapper.selectPowerTrendByDate(ThirdDailyAccumulationUpdateView.builder()
                .startTime(startTime)
                .endTime(endTime)
                .stationNameList(stationNameList)
                .systemSource(oilFieldCode)
                .dayFlag(dateType)
                .build());
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(powerList) && CollUtil.isNotEmpty(stationNameList)) {
            if (flag == 1) {
                powerMap = powerList.stream()
                        .collect(Collectors.toMap(ThirdDailyAccumulationUpdateView::getCreateDate, ThirdDailyAccumulationUpdateView::getPhotovoltaicPower));
            } else {
                powerMap = powerList.stream()
                        .collect(Collectors.toMap(ThirdDailyAccumulationUpdateView::getCreateDate, ThirdDailyAccumulationUpdateView::getDailyElectricityConsumption));
            }
        } else {
            powerMap = Collections.emptyMap();
        }
        return powerMap;
    }

    /**
     * 获取井口列表
     *
     * @param context 上下文
     * @param ouIds   ou-ids
     * @return {@link List }<{@link DynamicQueryBaseWellheadOutput }>
     */
    private List<DynamicQueryBaseWellheadOutput> getWellListByOuId(XContext context, List<String> ouIds) {
        IBaseWellheadCloudService service = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = service.queryBaseWellheadListByParam(context, DynamicQueryBaseWellheadInput.builder()
                .ouIds(ouIds)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取井口列表
     *
     * @param context    上下文
     * @param ouIds      ou-ids
     * @param runTypeKey 运行类型键
     * @return {@link List }<{@link DynamicQueryBaseWellheadOutput }>
     */
    private List<DynamicQueryBaseWellheadOutput> getWellListByOuId(XContext context, List<String> ouIds, String runTypeKey) {
        IBaseWellheadCloudService service = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = service.queryBaseWellheadListByParam(context, DynamicQueryBaseWellheadInput.builder()
                .ouIds(ouIds)
                .runTypeKey(runTypeKey)
                .build());
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 查询用电功率
     *
     * @param context      上下文
     * @param wellheadList 井口一览表
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    private Map<Date, BigDecimal> queryPowerConsumptionHour(XContext context, List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList, Date startTime, Date endTime) {
        Map<Date, BigDecimal> avgMap;
        if (CollUtil.isNotEmpty(wellheadList)) {
            ThirdWellAvgActivePowerViewMapper mapper = context.getBean(ThirdWellAvgActivePowerViewMapper.class);
            avgMap = mapper.selectPowerConsumptionHour(ThirdWellAvgActivePowerView.builder()
                            .wellNumbers(wellheadList.stream()
                                    .map(DynamicQueryBasePowerLineWellheadViewOutput::getWellNumber)
                                    .collect(Collectors.toList()))
                            .startTime(startTime)
                            .endTime(endTime)
                            .build())
                    .stream()
                    .collect(Collectors.toMap(ThirdWellAvgActivePowerView::getInputTime, ThirdWellAvgActivePowerView::getAvgActivePower));
        } else {
            avgMap = Collections.emptyMap();
        }
        return avgMap;
    }

    /**
     * 查询发电功率
     *
     * @param context   上下文
     * @param plantList 工厂清单
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    private Map<Date, BigDecimal> queryGeneratedPowerHour(XContext context, List<DynamicQueryBasePowerLinePlantViewOutput> plantList, Date startTime, Date endTime) {
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(plantList)) {
            ThirdActivePowerViewMapper dailyUpdateMapper = context.getBean(ThirdActivePowerViewMapper.class);
            powerMap = dailyUpdateMapper.selectGeneratedPowerHour(ThirdActivePowerView.builder()
                            .stationNames(plantList.stream()
                                    .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                                    .collect(Collectors.toList()))
                            .startTime(startTime)
                            .endTime(endTime)
                            .build())
                    .stream()
                    .collect(Collectors.toMap(ThirdActivePowerView::getCreateDate, ThirdActivePowerView::getPhotovoltaicPower));
        } else {
            powerMap = Collections.emptyMap();
        }
        return powerMap;
    }

    /**
     * 设置功率分析列表
     *
     * @param avgMap    平均地图
     * @param powerMap  功率图
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return {@link List }<{@link GetPowerAnalyseOutput }>
     */
    private List<GetPowerAnalyseOutput> setPowerAnalyseList(Map<Date, BigDecimal> avgMap, Map<Date, BigDecimal> powerMap, Date startTime, Date endTime) {
        //封装数据
        List<DateTime> rangeToList = DateUtil.rangeToList(startTime, endTime, DateField.HOUR_OF_DAY);
        List<GetPowerAnalyseOutput> outputs = new ArrayList<>(rangeToList.size());
        BigDecimal powerConsumption;
        BigDecimal powerGeneration;
        for (DateTime dateTime : rangeToList) {
            if (avgMap.containsKey(dateTime)) {
                powerConsumption = avgMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                powerConsumption = BigDecimal.ZERO;
            }
            if (powerMap.containsKey(dateTime)) {
                powerGeneration = powerMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                powerGeneration = BigDecimal.ZERO;
            }
            outputs.add(GetPowerAnalyseOutput.builder()
                    .dateFormat(dateTime.toString())
                    .powerConsumption(powerConsumption)
                    .powerGeneration(powerGeneration)
                    .build());
        }
        return outputs;
    }

    /**
     * 用电趋势统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link Map }<{@link Date }, {@link GetLineDailyElectricityTrendOutput }>
     */
    private Map<Date, GetLineDailyElectricityTrendOutput> getDailyElectricityTrendMap(XContext context, GetLineDailyElectricityTrendInput input) {
        IDailyElectricityTrendCloudService service = context.getBean(IDailyElectricityTrendCloudService.class);
        XListResult<GetLineDailyElectricityTrendOutput> result = service.queryDailyElectricityTrend(context, input);
        result.throwIfFail();
        List<GetLineDailyElectricityTrendOutput> list = result.getResult();
        Map<Date, GetLineDailyElectricityTrendOutput> collect;
        if (CollUtil.isNotEmpty(list)) {
            collect = list.stream()
                    .collect(Collectors.toMap(GetLineDailyElectricityTrendOutput::getCreateDate, Function.identity()));
        } else {
            collect = Collections.emptyMap();
        }
        return collect;
    }

    /**
     * 计算经济效益
     *
     * @param photovoltaicPower 光伏发电
     * @return {@link BigDecimal }
     */
    private BigDecimal calculateEconomicBenefit(BigDecimal photovoltaicPower) {
        BigDecimal economicBenefit;
        if (photovoltaicPower.compareTo(BigDecimal.ZERO) > 0) {
            economicBenefit = photovoltaicPower
                    .multiply(BigDecimal.valueOf(0.6))
                    .subtract(photovoltaicPower
                            .multiply(BigDecimal.valueOf(0.28)))
                    .setScale(2, RoundingMode.HALF_UP);
        } else {
            economicBenefit = BigDecimal.ZERO;
        }
        return economicBenefit;
    }

    /**
     * 计算累计减碳量
     *
     * @param photovoltaicPower 光伏发电
     * @return {@link BigDecimal }
     */
    private BigDecimal calculateCarbonReduction(BigDecimal photovoltaicPower) {
        BigDecimal carbonReduction;
        if (photovoltaicPower.compareTo(BigDecimal.ZERO) > 0) {
            carbonReduction = photovoltaicPower
                    .multiply(BusinessConstant.BIG_DECIMAL_6_67)
                    .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
        } else {
            carbonReduction = BigDecimal.ZERO;
        }
        return carbonReduction;
    }

    /**
     * 获取环比
     * (本期-上期) /上期*100%
     *
     * @param currentPeriod 本期
     * @param lastPeriod    上期
     * @return {@link BigDecimal}
     */
    private BigDecimal getCycleRatio(BigDecimal currentPeriod, BigDecimal lastPeriod) {
        BigDecimal cycleRatio;
        if (lastPeriod.compareTo(BigDecimal.ZERO) > 0) {
            cycleRatio = currentPeriod.subtract(lastPeriod)
                    .divide(lastPeriod, 4, RoundingMode.HALF_UP)
                    .multiply(BusinessConstant.BIG_DECIMAL_100);
        } else {
            cycleRatio = BigDecimal.ZERO;
        }
        return cycleRatio;
    }
}
