package pps.core.prediction.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.sys_organization.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.entity.PlantPredictedPowerLongTermDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataMapper;
import xstartup.base.XContext;
import xstartup.data.XListResult;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 统计分析基类
 *
 * @author ZWT
 * @date 2023/09/28 10:21
 */
public class StatisticAnalysisBaseService {

    /**
     * 获得光伏发电
     *
     * @param context     上下文
     * @param dateType    日期类型
     * @param plantIdList 工厂id列表
     * @param startTime   开始时间
     * @param endTime     结束时间
     * @return {@link List}
     */
    public List getPhotovoltaicPowerGeneration(XContext context, Integer dateType,
                                               List<String> plantIdList, Date startTime, Date endTime) {
        List powerGenerationList = null;
        //取当前时间
        DateTime date = DateUtil.beginOfDay(DateUtil.date());
        switch (dateType) {
            case 0:
                PlantPredictedPowerDataMapper powerDataMapper = context.getBean(PlantPredictedPowerDataMapper.class);
                powerGenerationList = powerDataMapper.selectList(new QueryWrapper<PlantPredictedPowerDataEnt>()
                        .select("hour_time",
                                "SUM( power ) AS power")
                        .lambda()
                        .eq(PlantPredictedPowerDataEnt::getYearTime, String.valueOf(date.year()))
                        .eq(PlantPredictedPowerDataEnt::getMonthTime, BaseUtils.getMonthString(date.month() + 1))
                        .eq(PlantPredictedPowerDataEnt::getDayTime, String.valueOf(date.dayOfMonth()))
                        .in(PlantPredictedPowerDataEnt::getPlantId, plantIdList)
                        .groupBy(PlantPredictedPowerDataEnt::getHourTime)
                        .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
                );
                break;
            case 1:
                powerGenerationList = this.getPlantPredictedPowerList(context, plantIdList,
                        date.toString(BusinessConstant.DATE_FORMAT_DAY),
                        DateUtil.offsetDay(date, 7).toString(BusinessConstant.DATE_FORMAT_DAY));
                break;
            case 2:
                powerGenerationList = this.getPlantPredictedPowerLongList(context, plantIdList,
                        date.toString(BusinessConstant.DATE_FORMAT_DAY),
                        DateUtil.offsetDay(date, 30).toString(BusinessConstant.DATE_FORMAT_DAY));
                break;
            case 3:
                //判断当前时间距当前时间是否大于30天
                if (Objects.isNull(startTime) || Objects.isNull(endTime)) {
                    return Collections.emptyList();
                }
                int between = (int) date.between(endTime, DateUnit.DAY);
                //判断结束时间是否大于当前时间30天以后
                if (endTime.compareTo(date) >= 0 && between >= 30) {
                    //长期预测
                    powerGenerationList = this.getPlantPredictedPowerLongList(context, plantIdList,
                            DateUtil.date(startTime).toString(BusinessConstant.DATE_FORMAT_DAY),
                            DateUtil.date(endTime).toString(BusinessConstant.DATE_FORMAT_DAY));
                } else {
                    //短期
                    powerGenerationList = this.getPlantPredictedPowerList(context, plantIdList,
                            DateUtil.date(startTime).toString(BusinessConstant.DATE_FORMAT_DAY),
                            DateUtil.date(endTime).toString(BusinessConstant.DATE_FORMAT_DAY));
                }
                break;
            default:
        }
        return powerGenerationList;
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    public List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 获取光伏发电厂
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link GetBasePhotovoltaicPlantCloudOutput}>
     */
    public List<GetBasePhotovoltaicPlantCloudOutput> getPhotovoltaicPlantByParam(XContext context, GetBasePhotovoltaicPlantCloudInput input) {
        BasePhotovoltaicPlantCloudService cloudService = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = cloudService.getBasePhotovoltaicPlantList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 获取短期预测结果
     *
     * @param context     上下文
     * @param plantIdList 工厂id列表
     * @param startTime   开始时间
     * @param endTime     结束时间
     * @return {@link List}<{@link PlantPredictedPowerDataEnt}>
     */
    private List<PlantPredictedPowerDataEnt> getPlantPredictedPowerList(XContext context, List<String> plantIdList,
                                                                        String startTime, String endTime) {
        PlantPredictedPowerDataMapper powerDataMapper = context.getBean(PlantPredictedPowerDataMapper.class);
        return powerDataMapper.selectList(new QueryWrapper<PlantPredictedPowerDataEnt>()
                .select("hour_time",
                        "SUM( power ) AS power")
                .lambda()
                .in(PlantPredictedPowerDataEnt::getPlantId, plantIdList)
                .between(PlantPredictedPowerDataEnt::getDataDate, startTime, endTime)
                .groupBy(PlantPredictedPowerDataEnt::getHourTime)
                .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
        );
    }

    /**
     * 获取长期预测结果
     *
     * @param context     上下文
     * @param plantIdList 工厂id列表
     * @param startTime   开始时间
     * @param endTime     结束时间
     * @return {@link List}<{@link PlantPredictedPowerLongTermDataEnt}>
     */
    private List<PlantPredictedPowerLongTermDataEnt> getPlantPredictedPowerLongList(XContext context, List<String> plantIdList,
                                                                                    String startTime, String endTime) {
        PlantPredictedPowerLongTermDataMapper longTermDataMapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
        return longTermDataMapper.selectList(new QueryWrapper<PlantPredictedPowerLongTermDataEnt>()
                .select("hour_time",
                        "SUM( power ) AS power")
                .lambda()
                .in(PlantPredictedPowerLongTermDataEnt::getPlantId, plantIdList)
                .between(PlantPredictedPowerLongTermDataEnt::getDataDate, startTime, endTime)
                .groupBy(PlantPredictedPowerLongTermDataEnt::getHourTime)
                .orderByAsc(PlantPredictedPowerLongTermDataEnt::getHourTime)
        );
    }
}
